#include <stdio.h>

#if defined __LCC__ && __LCC__ < 124

typedef struct {
	unsigned long long lo;
	unsigned long long hi;
} __uint128_t;

__uint128_t str2ulll(unsigned char *, unsigned char **, int);
unsigned long long udiv128(__uint128_t, unsigned long long, unsigned long long *);

unsigned long long Udiv128(__uint128_t a, unsigned long long b, unsigned long long *rem)
{
	unsigned long long q, r, carry, temp;
	int i;

	q = a.lo << 1;	// http://codereview.stackexchange.com/questions/67962
	r = a.hi;
	carry = a.lo >> 63;
	for (i = 0; i < 64; i++) {
		temp = r >> 63;
		r = r << 1 | carry;
		carry = temp;
		if (carry == 0) {
			if (r < b)
				goto shift;
			carry++;
		}
		r -= b;
	shift:	temp = q >> 63;
		q = q << 1 | carry;
		carry = temp;
	}
	*rem = r;
	return q;
}

int main(void)	// http://ttmath.org/online_calculator
{
	__uint128_t dividend;
	unsigned long long divisor, quotient, remainder;
	int r1, r2;
	unsigned char s[BUFSIZ];

	setvbuf(stdout, NULL, _IONBF, 0);
	printf("  Dividend: ");
	r1 = scanf("%40s", s);
	dividend = str2ulll(s, NULL, 10);
	printf("   Divisor: ");
	r2 = scanf("%llu", &divisor);
	// > 340282366920938463444927863358058659839 / 18446744073709551615
	if (dividend.hi >= divisor) {
		printf("Overflow!\n");
		return -1;
	}
	quotient = Udiv128(dividend, divisor, &remainder);
	printf("\nC quotient: %llu, remainder: %llu", quotient, remainder);
	remainder = -1;
	quotient = udiv128(dividend, divisor, &remainder);
	printf("\n  Quotient: %llu, remainder: %llu\n", quotient, remainder);
	return r1 - r2;
}

#else

#ifdef __ARC64__
# include <unistd.h>
#endif

#ifdef __amd64__
# define ATTRIB __attribute__((sysv_abi))
#else
# define ATTRIB
#endif

__uint128_t str2ulll(unsigned char *, unsigned char **, int);
unsigned long long ATTRIB udiv128(__uint128_t a, unsigned long long b, unsigned long long *rem);

int main(void)	// http://ttmath.org/online_calculator
{
	__uint128_t dividend;
	unsigned long long divisor, quotient, remainder;
	int r1, r2;
	unsigned char s[BUFSIZ];

	setvbuf(stdin, NULL, _IONBF, 0);
	setvbuf(stdout, NULL, _IONBF, 0);
	printf("  Dividend: ");
	r1 = scanf("%40s", s);
	dividend = str2ulll(s, NULL, 10);
	printf("   Divisor: ");
	r2 = scanf("%llu", &divisor);
	// > 340282366920938463444927863358058659839 / 18446744073709551615
	if (dividend >> 64 >= divisor) {
		printf("Overflow!\n");
#ifdef __ARC64__
		_exit(-1);
#else
		return -1;
#endif
	}
	quotient = (unsigned long long)(dividend / divisor);
	remainder = (unsigned long long)(dividend % divisor);
	printf("\nC quotient: %llu, remainder: %llu", quotient, remainder);
	remainder = -1;
	quotient = udiv128(dividend, divisor, &remainder);
	printf("\n  Quotient: %llu, remainder: %llu\n", quotient, remainder);
#ifdef __ARC64__
	_exit(r1 - r2);
#else
	return r1 - r2;
#endif
}

#endif
