#include <stdio.h>
#include <inttypes.h>

#if defined __LCC__ && __LCC__ < 124

typedef struct {
	uint64_t lo;
	uint64_t hi;
} __uint128_t;
typedef struct {
	__uint128_t lo;
	__uint128_t hi;
} __uint256_t;

__uint128_t str2ulll(unsigned char *, unsigned char **, int);
__uint256_t umul128(__uint128_t, __uint128_t);
__uint128_t udiv256(__uint128_t, __uint128_t, __uint128_t, __uint128_t *);

__uint128_t Udiv256(__uint128_t a_low, __uint128_t a_high, __uint128_t b, __uint128_t *rem)
{
	__uint128_t q, r;	// http://codereview.stackexchange.com/questions/67962
	uint64_t carry;
	int i;

	q.hi = a_low.hi << 1 | a_low.lo >> 63;
	q.lo = a_low.lo << 1;
	r.hi = a_high.hi;
	r.lo = a_high.lo;
	carry = a_low.hi >> 63;
	for (i = 0; i < 128; i++) {
		uint64_t temp = r.hi >> 63;
		r.hi = r.hi << 1 | r.lo >> 63;
		r.lo <<= 1;
		r.lo |= carry;
		carry = temp;
		if (carry == 0) {
			if (r.hi < b.hi || (r.hi == b.hi && r.lo < b.lo))
				goto shift;
			carry++;
		}
		r.hi -= b.hi;
		if (r.lo < b.lo)
			r.hi--;
		r.lo -= b.lo;
	shift:	temp = q.hi >> 63;
		q.hi = q.hi << 1 | q.lo >> 63;
		q.lo <<= 1;
		q.lo |= carry;
		carry = temp;
	}
	rem->hi = r.hi;
	rem->lo = r.lo;
	return q;
}

__uint128_t add(__uint128_t n, uint8_t addend)
{
	uint64_t old = n.lo;
	n.lo += addend;
	if (n.lo < old)
		n.hi++;
	return n;
}

__uint128_t x10(__uint128_t n)
{
	uint64_t m;
	n.hi = n.hi << 1 | n.lo >> 63;  // x2
	n.lo <<= 1;
	n.hi += n.hi << 2 | n.lo >> 62; // x5 (x1 + x4)
	m = n.lo;
	n.lo += n.lo << 2;
	if (n.lo < m)
		n.hi++;
	return n;
}

char *ulll2str(__uint128_t n, char *p)
{
	__uint128_t q = n;
	p += 40;
	*p = '\0';
	do {
		uint64_t x = q.lo;
		// H. Warren, Hacker's Delight, 10-14 Sample Magic Numbers
		__uint128_t m = {0xCCCCCCCCCCCCCCCD, 0xCCCCCCCCCCCCCCCC};
		__uint256_t y = umul128(q, m);
		q = y.hi;
		q.lo = q.hi << 61 | q.lo >> 3;
		q.hi >>= 3;
		*--p = (char)(x - 10 * q.lo + '0');
	} while (q.hi | q.lo);
	return p;
}

int main()	// http://ttmath.org/online_calculator
{
	__uint128_t thi, tmp, high, low, divisor, quotient, remainder;
	char c, s[BUFSIZ], t[BUFSIZ];
	int i;

	printf("  Dividend: ");
	fgets(s, sizeof(s), stdin);
	high.hi = high.lo = low.hi = low.lo = 0;
	for (i = 0; (c = s[i]) >= '0' && c <= '9'; i++) {
		// http://stackoverflow.com/questions/37756465
		// high *= 10;
		high = x10(high);
		// high += ((low >> 64) * 10 + ((low & -1ULL) * 10 >> 64)) >> 64;
		// (low >> 64) * 10
		thi.hi = 0;
		thi.lo = low.hi;
		thi = x10(thi);
		// (low & -1ULL) * 10
		tmp.hi = 0;
		tmp.lo = low.lo;
		tmp = x10(tmp);
		// + (... >> 64)
		thi.lo += tmp.hi;
		if (thi.lo < tmp.hi)
			thi.hi++;
		// high += ... >> 64;
		high.lo += thi.hi;
		if (high.lo < thi.hi)
			high.hi++;
		// low *= 10;
		low = x10(low);
		// tmp = low;
		tmp.lo = low.lo;
		tmp.hi = low.hi;
		// low += c - '0';
		low = add(low, (uint8_t)c - '0');
		// high += low < tmp;
		high = add(high, low.hi < tmp.hi || (low.hi == tmp.hi && low.lo < tmp.lo));
	}
	printf("   Divisor: ");
	divisor = str2ulll((uint8_t *)fgets(s, sizeof(s), stdin), NULL, 10);
	// > 115792089237316195423570985008687907852929702298719625575994209400481361428479 / 340282366920938463463374607431768211455
	if (high.hi > divisor.hi || (high.hi == divisor.hi && high.lo >= divisor.lo)) {
		printf("Overflow!\n");
		return -1;
	}
	quotient = Udiv256(low, high, divisor, &remainder);
	printf("\nC quotient: %s, remainder: %s", ulll2str(quotient, s), ulll2str(remainder, t));
	remainder.hi = remainder.lo = ~0ULL;
	quotient = udiv256(low, high, divisor, &remainder);
	printf("\n  Quotient: %s, remainder: %s\n", ulll2str(quotient, s), ulll2str(remainder, t));
	return 0;
}

#else

#ifdef __ARC64__
# include <unistd.h>
#endif

#ifdef __amd64__
# define ATTRIB __attribute__((sysv_abi))
#else
# define ATTRIB
#endif

__uint128_t str2ulll(unsigned char *, unsigned char **, int);
__uint128_t ATTRIB udiv256(__uint128_t a_lo, __uint128_t a_hi, __uint128_t b, __uint128_t *rem);

__uint128_t Udiv256(__uint128_t a_lo, __uint128_t a_hi, __uint128_t b, __uint128_t *rem)
{
	__uint128_t q, r, carry, temp;
	int i;

	q = a_lo << 1;	// http://codereview.stackexchange.com/questions/67962
	r = a_hi;
	carry = a_lo >> 127;
	for (i = 0; i < 128; i++) {
		temp = r >> 127;
		r = r << 1 | carry;
		carry = temp;
		if (carry == 0) {
			if (r < b)
				goto shift;
			carry++;
		}
		r -= b;
	shift:	temp = q >> 127;
		q = q << 1 | carry;
		carry = temp;
	}
	*rem = r;
	return q;
}

char *ulll2str(char *p, __uint128_t n)
{
	*p = '\0';
	do
		*--p = n % 10 + '0';
	while (n /= 10);
	return p;
}

int main(void)	// http://ttmath.org/online_calculator
{
	__uint128_t tmp, high = 0, low = 0, divisor, quotient, remainder;
	int i;
	char s[BUFSIZ], c;

	setvbuf(stdin, NULL, _IONBF, 0);
	setvbuf(stdout, NULL, _IONBF, 0);
	printf("  Dividend: ");
	fgets(s, sizeof(s), stdin);
	for (i = 0; (c = s[i]) >= '0' && c <= '9'; i++) {
		high *= 10;	// http://stackoverflow.com/questions/37756465
		high += ((low >> 64) * 10 + ((low & -1ULL) * 10 >> 64)) >> 64;
		low *= 10;
		tmp = low;
		low += c - '0';
		high += low < tmp;
	}
	printf("   Divisor: ");
	divisor = str2ulll((uint8_t *)fgets(s, sizeof(s), stdin), NULL, 10);
	//> 115792089237316195423570985008687907852929702298719625575994209400481361428479 / 340282366920938463463374607431768211455
	if (high >= divisor) {
		printf("Overflow!\n");
#ifdef __ARC64__
		_exit(-1);
#else
		return -1;
#endif
	}
	quotient = Udiv256(low, high, divisor, &remainder);
	printf("\nC quotient: ");
	printf("%s, remainder: ", ulll2str(s + sizeof(s) - 1, quotient));
	puts(ulll2str(s + sizeof(s) - 1, remainder));
	remainder = ~0;
	quotient = udiv256(low, high, divisor, &remainder);
	printf("  Quotient: ");
	printf("%s, remainder: ", ulll2str(s + sizeof(s) - 1, quotient));
	puts(ulll2str(s + sizeof(s) - 1, remainder));
#ifdef __ARC64__
	_exit(0);
#else
	return 0;
#endif
}

#endif
